@@ -1,19 +1,4 @@
-/*
-   Copyright (C) 2013 Simo Sorce <simo@samba.org>
-
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Lesser General Public
-   License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Lesser General Public License for more details.
-
-   You should have received a copy of the GNU Lesser General Public
-   License along with this library; if not, see <http://www.gnu.org/licenses/>.
-*/
+/* Copyright 2013 Simo Sorce <simo@samba.org>, see COPYING for license */
 
 #ifndef _GSS_NTLMSSP_H_
 #define _GSS_NTLMSSP_H_
@@ -26,17 +11,6 @@
 #define DEF_NB_DOMAIN "WORKSTATION"
 #define MAX_CHALRESP_LIFETIME 36 * 60 * 60 /* 36 hours in seconds */
 
-#define SEC_LEVEL_MIN 0
-#define SEC_LEVEL_MAX 5
-
-#define SEC_LM_OK 0x01
-#define SEC_NTLM_OK 0x02
-#define SEC_EXT_SEC_OK 0x04
-#define SEC_V2_ONLY 0x08
-#define SEC_DC_LM_OK 0x10
-#define SEC_DC_NTLM_OK 0x20
-#define SEC_DC_V2_OK 0x40
-
 #define NTLMSSP_DEFAULT_CLIENT_FLAGS ( \
                 NTLMSSP_NEGOTIATE_ALWAYS_SIGN | \
                 NTLMSSP_NEGOTIATE_128 | \
@@ -44,7 +18,8 @@
                 NTLMSSP_NEGOTIATE_NTLM | \
                 NTLMSSP_REQUEST_TARGET | \
                 NTLMSSP_NEGOTIATE_OEM | \
-                NTLMSSP_NEGOTIATE_UNICODE)
+                NTLMSSP_NEGOTIATE_UNICODE | \
+                NTLMSSP_NEGOTIATE_VERSION)
 
 #define NTLMSSP_DEFAULT_SERVER_FLAGS ( \
                 NTLMSSP_NEGOTIATE_ALWAYS_SIGN | \
@@ -66,6 +41,11 @@
 #define NTLMSSP_CTX_FLAG_SPNEGO_CAN_MIC 0x02 /* SPNEGO asks for MIC */
 #define NTLMSSP_CTX_FLAG_AUTH_WITH_MIC  0x04 /* Auth MIC was created */
 
+struct gssntlm_name_attribute {
+    char *attr_name; /* NULL indicates array termination */
+    gss_buffer_desc attr_value;
+};
+
 struct gssntlm_name {
     enum ntlm_name_type {
         GSSNTLM_NAME_NULL,
@@ -80,9 +60,12 @@ struct gssntlm_name {
             char *name;
         } user;
         struct {
+            char *spn;
             char *name;
         } server;
     } data;
+
+    struct gssntlm_name_attribute *attrs; /* Array of name attributes */
 };
 
 struct gssntlm_cred {
@@ -105,11 +88,19 @@ struct gssntlm_cred {
         } user;
         struct {
             struct gssntlm_name name;
+            char *keyfile;
         } server;
         struct {
             struct gssntlm_name user;
+            bool creds_in_cache;
         } external;
     } cred;
+
+    /* set cred options provided default flags
+     * this is currently intentionally not imported/exported
+     * as it is considered an ephemeral local status
+     */
+    uint32_t neg_flags;
 };
 
 struct gssntlm_ctx {
@@ -148,13 +139,13 @@ struct gssntlm_ctx {
     /* negotiated flags */
     uint32_t neg_flags;
 
-    /* TODO: Add whitelist of servers we are allowed to communicate with */
-
     struct ntlm_key exported_session_key;
     struct ntlm_signseal_state crypto_state;
 
     uint32_t int_flags;
     time_t expiration_time;
+
+    void *external_context;
 };
 
 #define set_GSSERRS(min, maj) \
@@ -172,7 +163,10 @@ static inline uint32_t gssntlmssp_ret_err(uint32_t *s, uint32_t n, uint32_t j)
     DEBUG_GSS_ERRORS((retmaj = (maj)), (retmin = (min))) ? 0 : \
      gssntlmssp_ret_err(minor_status, retmin, retmaj)
 
-uint8_t gssntlm_required_security(int security_level, struct gssntlm_ctx *ctx);
+/* Static const name attribute for sids list */
+extern const char gssntlmssp_sids_urn[];
+
+bool gssntlm_required_security(int security_level, struct gssntlm_ctx *ctx);
 
 void gssntlm_set_role(struct gssntlm_ctx *ctx,
                       int desired, char *nb_domain_name);
@@ -182,6 +176,7 @@ bool gssntlm_role_is_domain_member(struct gssntlm_ctx *ctx);
 
 bool gssntlm_sec_lm_ok(struct gssntlm_ctx *ctx);
 bool gssntlm_sec_ntlm_ok(struct gssntlm_ctx *ctx);
+bool gssntlm_sec_v2_ok(struct gssntlm_ctx *ctx);
 bool gssntlm_ext_sec_ok(struct gssntlm_ctx *ctx);
 
 uint32_t gssntlm_context_is_valid(struct gssntlm_ctx *ctx,
@@ -189,14 +184,31 @@ uint32_t gssntlm_context_is_valid(struct gssntlm_ctx *ctx,
 
 int gssntlm_get_lm_compatibility_level(void);
 
+uint32_t gssntlm_mech_invoke(uint32_t *minor_status,
+                             const gss_OID desired_mech,
+                             const gss_OID desired_object,
+                             gss_buffer_t value);
+
 void gssntlm_int_release_name(struct gssntlm_name *name);
 void gssntlm_int_release_cred(struct gssntlm_cred *cred);
 
+size_t gssntlm_get_attrs_count(const struct gssntlm_name_attribute *attrs);
+int gssntlm_copy_attrs(const struct gssntlm_name_attribute *src,
+                       struct gssntlm_name_attribute **dst);
+struct gssntlm_name_attribute *gssntlm_find_attr(
+                                        struct gssntlm_name_attribute *attrs,
+                                        const char *attr_name,
+                                        size_t attr_name_len);
+void gssntlm_release_attrs(struct gssntlm_name_attribute **attrs);
+
 int gssntlm_copy_name(struct gssntlm_name *src, struct gssntlm_name *dst);
 int gssntlm_copy_creds(struct gssntlm_cred *in, struct gssntlm_cred *out);
 
-uint32_t external_netbios_get_names(char **computer, char **domain);
-uint32_t external_get_creds(struct gssntlm_name *name,
+void *external_get_context(void);
+void external_free_context(void *ctx);
+uint32_t external_netbios_get_names(void *ctx, char **computer, char **domain);
+uint32_t external_get_creds(void *ctx,
+                            struct gssntlm_name *name,
                             struct gssntlm_cred *cred);
 uint32_t external_cli_auth(struct gssntlm_ctx *ctx,
                            struct gssntlm_cred *cred,
@@ -208,7 +220,7 @@ uint32_t external_srv_auth(struct gssntlm_ctx *ctx,
                            struct ntlm_buffer *lm_chal_resp,
                            struct ntlm_key *session_base_key);
 
-uint32_t netbios_get_names(char *computer_name,
+uint32_t netbios_get_names(void *ctx, char *computer_name,
                            char **netbios_host, char **netbios_domain);
 
 bool is_ntlm_v1(struct ntlm_buffer *nt_chal_resp);
@@ -238,6 +250,7 @@ uint32_t gssntlm_acquire_cred(uint32_t *minor_status,
                               uint32_t *time_rec);
 
 uint32_t gssntlm_acquire_cred_from(uint32_t *minor_status,
+                                   void *external_context,
                                    gss_name_t desired_name,
                                    uint32_t time_req,
                                    gss_OID_set desired_mechs,
@@ -416,9 +429,38 @@ uint32_t gssntlm_display_status(uint32_t *minor_status,
 				uint32_t *message_context,
 				gss_buffer_t status_string);
 
+uint32_t gssntlm_get_name_attribute(uint32_t *minor_status,
+                                    gss_name_t name,
+                                    gss_buffer_t attr,
+                                    int *authenticated,
+                                    int *complete,
+                                    gss_buffer_t value,
+                                    gss_buffer_t display_value,
+                                    int *more);
+
 uint32_t gssntlm_inquire_name(uint32_t *minor_status,
                               gss_name_t name,
                               int *name_is_MN,
                               gss_OID *MN_mech,
                               gss_buffer_set_t *attrs);
+
+uint32_t gssntlm_inquire_saslname_for_mech(OM_uint32 *minor_status,
+                                           const gss_OID desired_mech,
+                                           gss_buffer_t sasl_mech_name,
+                                           gss_buffer_t mech_name,
+                                           gss_buffer_t mech_description);
+
+uint32_t gssntlm_inquire_mech_for_saslname(OM_uint32 *minor_status,
+                                           const gss_buffer_t sasl_mech_name,
+                                           gss_OID *mech_type);
+
+uint32_t gssntlm_inquire_attrs_for_mech(uint32_t *minor_status,
+                                        gss_const_OID mech_oid,
+                                        gss_OID_set *mech_attrs,
+                                        gss_OID_set *known_mech_attrs);
+
+uint32_t gssntlm_set_cred_option(uint32_t *minor_status,
+                                 gss_cred_id_t *cred_handle,
+                                 const gss_OID desired_object,
+                                 const gss_buffer_t value);
 #endif /* _GSS_NTLMSSP_H_ */
