@@ -1,20 +1,6 @@
-/*
-   Copyright (C) 2013 Simo Sorce <simo@samba.org>
-
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Lesser General Public
-   License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Lesser General Public License for more details.
-
-   You should have received a copy of the GNU Lesser General Public
-   License along with this library; if not, see <http://www.gnu.org/licenses/>.
-*/
+/* Copyright 2013 Simo Sorce <simo@samba.org>, see COPYING for license */
 
+#include <endian.h>
 #include <errno.h>
 #include <stdlib.h>
 #include <string.h>
@@ -40,7 +26,6 @@ uint32_t gssntlm_init_sec_context(uint32_t *minor_status,
     struct gssntlm_ctx *ctx;
     struct gssntlm_name *server = NULL;
     struct gssntlm_cred *cred = NULL;
-    char *computer_name = NULL;
     char *nb_computer_name = NULL;
     char *nb_domain_name = NULL;
     struct gssntlm_name *client_name = NULL;
@@ -95,6 +80,11 @@ uint32_t gssntlm_init_sec_context(uint32_t *minor_status,
             set_GSSERRS(ERR_NOARG, GSS_S_CRED_UNAVAIL);
             goto done;
         }
+        if (cred->type == GSSNTLM_CRED_EXTERNAL &&
+            cred->cred.external.creds_in_cache == 0) {
+            set_GSSERRS(ERR_NOARG, GSS_S_CRED_UNAVAIL);
+            goto done;
+        }
     }
 
     if (ctx == NULL) {
@@ -106,6 +96,8 @@ uint32_t gssntlm_init_sec_context(uint32_t *minor_status,
             goto done;
         }
 
+        ctx->external_context = external_get_context();
+
         retmin = gssntlm_copy_name(&cred->cred.user.user,
                                    &ctx->source_name);
         if (retmin) {
@@ -124,6 +116,10 @@ uint32_t gssntlm_init_sec_context(uint32_t *minor_status,
         ctx->gss_flags = req_flags;
 
         ctx->neg_flags = NTLMSSP_DEFAULT_CLIENT_FLAGS;
+        /* override neg_flags default if requested */
+        if (cred->neg_flags) {
+            ctx->neg_flags = cred->neg_flags;
+        }
 
         /*
          * we ignore unsupported flags for now
@@ -139,7 +135,8 @@ uint32_t gssntlm_init_sec_context(uint32_t *minor_status,
         if ((req_flags & GSS_C_INTEG_FLAG) ||
             (req_flags & GSS_C_REPLAY_FLAG) ||
             (req_flags & GSS_C_SEQUENCE_FLAG)) {
-            ctx->neg_flags |= NTLMSSP_NEGOTIATE_SIGN;
+            ctx->neg_flags |= NTLMSSP_NEGOTIATE_SIGN |
+                              NTLMSSP_NEGOTIATE_KEY_EXCH;
         }
         if (req_flags & GSS_C_CONF_FLAG) {
             ctx->neg_flags |= NTLMSSP_NEGOTIATE_SEAL |
@@ -171,13 +168,8 @@ uint32_t gssntlm_init_sec_context(uint32_t *minor_status,
             if (retmaj) goto done;
         }
 
-        computer_name = strdup(client_name->data.server.name);
-        if (!computer_name) {
-            set_GSSERR(ENOMEM);
-            goto done;
-        }
-
-        retmin = netbios_get_names(computer_name,
+        retmin = netbios_get_names(ctx->external_context,
+                                   client_name->data.server.name,
                                    &nb_computer_name, &nb_domain_name);
         if (retmin) {
             set_GSSERR(retmin);
@@ -193,8 +185,7 @@ uint32_t gssntlm_init_sec_context(uint32_t *minor_status,
         gssntlm_set_role(ctx, GSSNTLM_CLIENT, nb_domain_name);
 
         lm_compat_lvl = gssntlm_get_lm_compatibility_level();
-        ctx->sec_req = gssntlm_required_security(lm_compat_lvl, ctx);
-        if (ctx->sec_req == 0xff) {
+        if (!gssntlm_required_security(lm_compat_lvl, ctx)) {
             set_GSSERR(ERR_BADLMLVL);
             goto done;
         }
@@ -405,7 +396,7 @@ uint32_t gssntlm_init_sec_context(uint32_t *minor_status,
             ctx->gss_flags |= GSS_C_INTEG_FLAG;
         }
         if (ctx->neg_flags & NTLMSSP_NEGOTIATE_SEAL) {
-            ctx->gss_flags |= GSS_C_CONF_FLAG & GSS_C_INTEG_FLAG;
+            ctx->gss_flags |= GSS_C_CONF_FLAG | GSS_C_INTEG_FLAG;
         }
 
         ctx->stage = NTLMSSP_STAGE_DONE;
@@ -440,7 +431,6 @@ done:
         gssntlm_release_cred(&tmpmin, (gss_cred_id_t *)&cred);
     }
     gssntlm_release_name(&tmpmin, (gss_name_t *)&client_name);
-    safefree(computer_name);
     safefree(nb_computer_name);
     safefree(nb_domain_name);
     safefree(trgt_name);
@@ -483,8 +473,9 @@ uint32_t gssntlm_delete_sec_context(uint32_t *minor_status,
     gssntlm_int_release_name(&ctx->source_name);
     gssntlm_int_release_name(&ctx->target_name);
 
-    RC4_FREE(&ctx->crypto_state.send.seal_handle);
-    RC4_FREE(&ctx->crypto_state.recv.seal_handle);
+    ntlm_release_rc4_state(&ctx->crypto_state);
+
+    external_free_context(ctx->external_context);
 
     safezero((uint8_t *)ctx, sizeof(struct gssntlm_ctx));
     safefree(*context_handle);
@@ -534,11 +525,10 @@ uint32_t gssntlm_accept_sec_context(uint32_t *minor_status,
                                     gss_cred_id_t *delegated_cred_handle)
 {
     struct gssntlm_ctx *ctx;
-    struct gssntlm_cred *cred;
+    struct gssntlm_cred *cred = NULL;
     int lm_compat_lvl = -1;
     struct ntlm_buffer challenge = { 0 };
     struct gssntlm_name *server_name = NULL;
-    char *computer_name = NULL;
     char *nb_computer_name = NULL;
     char *nb_domain_name = NULL;
     char *chal_target_name;
@@ -603,6 +593,8 @@ uint32_t gssntlm_accept_sec_context(uint32_t *minor_status,
             goto done;
         }
 
+        ctx->external_context = external_get_context();
+
         /* acquire our own name */
         if (!server_name) {
             gss_buffer_desc tmpbuf;
@@ -622,13 +614,8 @@ uint32_t gssntlm_accept_sec_context(uint32_t *minor_status,
             goto done;
         }
 
-        computer_name = strdup(server_name->data.server.name);
-        if (!computer_name) {
-            set_GSSERR(ENOMEM);
-            goto done;
-        }
-
-        retmin = netbios_get_names(computer_name,
+        retmin = netbios_get_names(ctx->external_context,
+                                   server_name->data.server.name,
                                    &nb_computer_name, &nb_domain_name);
         if (retmin) {
             set_GSSERR(retmin);
@@ -644,8 +631,7 @@ uint32_t gssntlm_accept_sec_context(uint32_t *minor_status,
         gssntlm_set_role(ctx, GSSNTLM_SERVER, nb_domain_name);
 
         lm_compat_lvl = gssntlm_get_lm_compatibility_level();
-        ctx->sec_req = gssntlm_required_security(lm_compat_lvl, ctx);
-        if (ctx->sec_req == 0xff) {
+        if (!gssntlm_required_security(lm_compat_lvl, ctx)) {
             set_GSSERR(ERR_BADLMLVL);
             goto done;
         }
@@ -653,6 +639,11 @@ uint32_t gssntlm_accept_sec_context(uint32_t *minor_status,
         ctx->neg_flags = NTLMSSP_DEFAULT_SERVER_FLAGS;
         /* Fixme: How do we allow anonymous negotition ? */
 
+        /* override neg_flags default if requested */
+        if (cred && cred->neg_flags) {
+            ctx->neg_flags = cred->neg_flags;
+        }
+
         if (gssntlm_sec_lm_ok(ctx)) {
             ctx->neg_flags |= NTLMSSP_REQUEST_NON_NT_SESSION_KEY;
             ctx->neg_flags |= NTLMSSP_NEGOTIATE_LM_KEY;
@@ -691,14 +682,24 @@ uint32_t gssntlm_accept_sec_context(uint32_t *minor_status,
 
             /* leave only the crossing between requested and allowed flags */
             ctx->neg_flags &= in_flags;
+
+            /* Try to force the use of NTLMSSP_NEGOTIATE_VERSION even if the
+             * client did not advertize it in their negotiate message, but
+             * should be capable of providing it.
+             * This is what Windows Server 2022 also does, and addresses
+             * issues with older clients that incorrectly deal with MIC
+             * calculations in absence of this flag. */
+            if ((ctx->neg_flags & (NTLMSSP_NEGOTIATE_ALWAYS_SIGN |
+                                    NTLMSSP_NEGOTIATE_SEAL |
+                                    NTLMSSP_NEGOTIATE_SIGN))) {
+                ctx->neg_flags |= NTLMSSP_NEGOTIATE_VERSION;
+            }
         } else {
             /* If there is no negotiate message set datagram mode */
             ctx->neg_flags |= NTLMSSP_NEGOTIATE_DATAGRAM | \
                               NTLMSSP_NEGOTIATE_KEY_EXCH;
         }
 
-        /* TODO: Support MS-NLMP ServerBlock ? */
-
         /* TODO: Check some minimum required flags ? */
         /* TODO: Check MS-NLMP ServerRequire128bitEncryption */
 
@@ -738,15 +739,27 @@ uint32_t gssntlm_accept_sec_context(uint32_t *minor_status,
             goto done;
         }
 
+        /* TODO: allow client applications to set a context option to
+         * provide an av_flags default value so that flags like
+         * MSVAVFLAGS_UNVERIFIED_SPN can be set. By default SSPI does
+         * not set this flag, and setting it causes servers with restrictive
+         * policy to fail authentication. Given no MS client set this flag
+         * by default, neither should we until there is a clear need. We
+         * trust our calling applications to do the right thing here.
+         * av_flags = MSVAVFLAGS_UNVERIFIED_SPN;
+         */
+
         timestamp = ntlm_timestamp_now();
 
         retmin = ntlm_encode_target_info(ctx->ntlm,
                                          nb_computer_name,
                                          nb_domain_name,
-                                         computer_name,
+                                         server_name->data.server.name,
                                          NULL, NULL,
-                                         NULL, &timestamp,
-                                         NULL, NULL, NULL,
+                                         &av_flags, &timestamp,
+                                         NULL,
+                                         server_name->data.server.spn,
+                                         NULL,
                                          &target_info);
         if (retmin) {
             set_GSSERR(retmin);
@@ -842,20 +855,27 @@ uint32_t gssntlm_accept_sec_context(uint32_t *minor_status,
             goto done;
         }
 
-        if (((usr_name == NULL) || (usr_name[0] == '\0')) &&
-            (nt_chal_resp.length == 0) &&
-            (((lm_chal_resp.length == 1) && (lm_chal_resp.data[0] == '\0')) ||
-             (lm_chal_resp.length == 0))) {
-            /* Anonymous auth */
-            /* FIXME: not supported for now */
-            set_GSSERR(ERR_NOTSUPPORTED);
-            goto done;
-
+        if ((usr_name == NULL) || (usr_name[0] == '\0')) {
+            if ((nt_chal_resp.length == 0) &&
+                (((lm_chal_resp.length == 1) &&
+                  (lm_chal_resp.data[0] == '\0')) ||
+                 (lm_chal_resp.length == 0))) {
+                /* Anonymous auth */
+                /* FIXME: not supported for now */
+                set_GSSERR(ERR_NOTSUPPORTED);
+                goto done;
+            } else {
+                set_GSSERR(ERR_NOUSRFOUND);
+                goto done;
+            }
         } else {
 
             char useratdom[1024];
             size_t ulen, dlen, uadlen;
             gss_buffer_desc usrname;
+            gss_const_key_value_set_t cred_store = GSS_C_NO_CRED_STORE;
+            gss_key_value_set_desc cs;
+            gss_key_value_element_desc cs_el;
 
             if (!dom_name) {
                 dom_name = strdup("");
@@ -865,20 +885,28 @@ uint32_t gssntlm_accept_sec_context(uint32_t *minor_status,
                 }
             }
 
+            /* Use domain\username format as that allows to pass in
+             * enterprise names without the need to escape them */
             ulen = strlen(usr_name);
             dlen = strlen(dom_name);
             if (ulen + dlen + 2 > 1024) {
                 set_GSSERR(ERR_NAMETOOLONG);
                 goto done;
             }
-            strncpy(useratdom, usr_name, ulen);
-            uadlen = ulen;
+            uadlen = dlen;
             if (dlen) {
-                useratdom[uadlen] = '@';
-                uadlen++;
-                strncpy(&useratdom[uadlen], dom_name, dlen);
-                uadlen += dlen;
+                memcpy(useratdom, dom_name, dlen);
             }
+
+            /* always add the domain separator, this way if the username
+             * is an enteprise name (user@email.domain form) it will be
+             * correctly recognized by gssntlm_import_name() as such */
+            useratdom[uadlen] = '\\';
+            uadlen++;
+
+            /* finally add usernmae part */
+            memcpy(&useratdom[uadlen], usr_name, ulen);
+            uadlen += ulen;
             useratdom[uadlen] = '\0';
 
             usrname.value = useratdom;
@@ -888,13 +916,22 @@ uint32_t gssntlm_accept_sec_context(uint32_t *minor_status,
                                          (gss_name_t *)&gss_usrname);
             if (retmaj) goto done;
 
-            retmaj = gssntlm_acquire_cred(&retmin,
-                                          (gss_name_t)gss_usrname,
-                                          GSS_C_INDEFINITE,
-                                          GSS_C_NO_OID_SET,
-                                          GSS_C_INITIATE,
-                                          (gss_cred_id_t *)&usr_cred,
-                                          NULL, NULL);
+            if (cred && cred->cred.server.keyfile) {
+                cs_el.key = GSS_NTLMSSP_CS_KEYFILE;
+                cs_el.value = cred->cred.server.keyfile;
+                cs.count = 1;
+                cs.elements = &cs_el;
+                cred_store = &cs;
+            }
+
+            retmaj = gssntlm_acquire_cred_from(&retmin, ctx->external_context,
+                                               (gss_name_t)gss_usrname,
+                                                GSS_C_INDEFINITE,
+                                                GSS_C_NO_OID_SET,
+                                                GSS_C_INITIATE,
+                                                cred_store,
+                                                (gss_cred_id_t *)&usr_cred,
+                                                NULL, NULL);
             if (retmaj) goto done;
             /* We can't handle winbind credentials yet */
             if (usr_cred->type != GSSNTLM_CRED_USER &&
@@ -942,6 +979,7 @@ uint32_t gssntlm_accept_sec_context(uint32_t *minor_status,
         }
 
         if (input_chan_bindings != GSS_C_NO_CHANNEL_BINDINGS) {
+            uint8_t zero_cb[16] = { 0 };
             if (input_chan_bindings->initiator_addrtype != 0 ||
                 input_chan_bindings->initiator_address.length != 0 ||
                 input_chan_bindings->acceptor_addrtype != 0 ||
@@ -953,12 +991,23 @@ uint32_t gssntlm_accept_sec_context(uint32_t *minor_status,
             unhashed_cb.length = input_chan_bindings->application_data.length;
             unhashed_cb.data = input_chan_bindings->application_data.value;
 
-            /* TODO: optionally allow to ignore CBT if av_cb is null ? */
-            retmin = ntlm_verify_channel_bindings(&unhashed_cb, &av_cb);
-            if (retmin) {
+            if (av_cb.length && (av_cb.length != 16)) {
                 set_GSSERRS(retmin, GSS_S_DEFECTIVE_TOKEN);
                 goto done;
             }
+            if (av_cb.length &&
+                (memcmp(av_cb.data, zero_cb, 16) != 0)) {
+                retmin = ntlm_verify_channel_bindings(&unhashed_cb, &av_cb);
+                if (retmin) {
+                    set_GSSERRS(retmin, GSS_S_DEFECTIVE_TOKEN);
+                    goto done;
+/* This flag has been introduced only recently in MIT krb5 */
+#ifdef GSS_C_CHANNEL_BOUND_FLAG
+                } else {
+                    ctx->gss_flags |= GSS_C_CHANNEL_BOUND_FLAG;
+#endif /* GSS_C_CHANNEL_BOUND_FLAG */
+                }
+            }
         }
 
         if (ctx->neg_flags & (NTLMSSP_NEGOTIATE_SIGN |
@@ -997,7 +1046,8 @@ done:
     }
     *context_handle = (gss_ctx_id_t)ctx;
     gssntlm_release_name(&tmpmin, (gss_name_t *)&server_name);
-    safefree(computer_name);
+    gssntlm_release_name(&tmpmin, (gss_name_t *)&gss_usrname);
+    gssntlm_release_cred(&tmpmin, (gss_cred_id_t *)&usr_cred);
     safefree(nb_computer_name);
     safefree(nb_domain_name);
     safefree(usr_name);
@@ -1219,6 +1269,71 @@ uint32_t gssntlm_spnego_req_mic(uint32_t *minor_status,
     return GSSERRS(retmin, retmaj);
 }
 
+static const gss_OID_desc sasl_ssf_oid = {
+    11, discard_const("\x2a\x86\x48\x86\xf7\x12\x01\x02\x02\x05\x0f")
+};
+
+static uint32_t gssntlm_sasl_ssf(uint32_t *minor_status,
+                                 struct gssntlm_ctx *ctx,
+                                 gss_buffer_set_t *data_set)
+{
+    uint32_t retmin;
+    uint32_t retmaj;
+    uint32_t tmpmin;
+    gss_buffer_desc ssf_buf;
+    uint32_t ssf = 0;
+
+    /* Handwaving a bit here but this is what SSF is all about */
+    if (ctx->neg_flags & NTLMSSP_NEGOTIATE_SEAL) {
+        if (ctx->neg_flags & NTLMSSP_NEGOTIATE_128) {
+            /* Technically we use RC4 with a 128 bit key, but we
+             * consider the RC4 strenght degraded so we assign
+             * it a value of 64, this is consistent with what
+             * the krb5 mechanism does for the Rc4-HMAC enctype */
+            ssf = 64;
+        } else if (ctx->neg_flags & NTLMSSP_NEGOTIATE_56) {
+            ssf = 56;
+        } else {
+            ssf = 40;
+        }
+    } else if (ctx->neg_flags & NTLMSSP_NEGOTIATE_SIGN) {
+        ssf = 1;
+    }
+
+    ssf = htobe32(ssf);
+    ssf_buf.value = &ssf;
+    ssf_buf.length = 4;
+
+    retmaj = gss_add_buffer_set_member(&retmin, &ssf_buf, data_set);
+    if (retmaj != GSS_S_COMPLETE) {
+        (void)gss_release_buffer_set(&tmpmin, data_set);
+    }
+
+    return GSSERRS(retmin, retmaj);
+}
+
+static uint32_t gssntlm_sspi_session_key(uint32_t *minor_status,
+                                         struct gssntlm_ctx *ctx,
+                                         gss_buffer_set_t *data_set)
+{
+    uint32_t retmin;
+    uint32_t retmaj;
+    uint32_t tmpmin;
+    gss_buffer_desc session_key_buf;
+
+    if (ctx->exported_session_key.length == 0) {
+      return GSSERRS(ERR_NOTAVAIL, GSS_S_UNAVAILABLE);
+    }
+
+    session_key_buf.length = ctx->exported_session_key.length;
+    session_key_buf.value = ctx->exported_session_key.data;
+
+    retmaj = gss_add_buffer_set_member(&retmin, &session_key_buf,  data_set);
+    if (retmaj != GSS_S_COMPLETE) {
+        (void)gss_release_buffer_set(&tmpmin, data_set);
+    }
+    return GSSERRS(retmin, retmaj);
+}
 
 uint32_t gssntlm_inquire_sec_context_by_oid(uint32_t *minor_status,
 	                                    const gss_ctx_id_t context_handle,
@@ -1244,6 +1359,10 @@ uint32_t gssntlm_inquire_sec_context_by_oid(uint32_t *minor_status,
 
     if (gss_oid_equal(desired_object, &spnego_req_mic_oid)) {
         return gssntlm_spnego_req_mic(minor_status, ctx, data_set);
+    } else if (gss_oid_equal(desired_object, &sasl_ssf_oid)){
+        return gssntlm_sasl_ssf(minor_status, ctx, data_set);
+    } else if (gss_oid_equal(desired_object, GSS_C_INQ_SSPI_SESSION_KEY)) {
+      return gssntlm_sspi_session_key(minor_status, ctx, data_set);
     }
 
     return GSSERRS(ERR_NOTSUPPORTED, GSS_S_UNAVAILABLE);
