@@ -1,19 +1,4 @@
-/*
-   Copyright (C) 2013 Simo Sorce <simo@samba.org>
-
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Lesser General Public
-   License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Lesser General Public License for more details.
-
-   You should have received a copy of the GNU Lesser General Public
-   License along with this library; if not, see <http://www.gnu.org/licenses/>.
-*/
+/* Copyright 2013 Simo Sorce <simo@samba.org>, see COPYING for license */
 
 #include <gssapi/gssapi.h>
 #include <gssapi/gssapi_ext.h>
@@ -68,7 +53,7 @@ OM_uint32 gss_acquire_cred_from(OM_uint32 *minor_status,
                                 gss_OID_set *actual_mechs,
                                 OM_uint32 *time_rec)
 {
-    return gssntlm_acquire_cred_from(minor_status,
+    return gssntlm_acquire_cred_from(minor_status, NULL,
                                      desired_name,
                                      time_req,
                                      desired_mechs,
@@ -162,6 +147,18 @@ OM_uint32 gss_release_name(OM_uint32 *minor_status,
     return gssntlm_release_name(minor_status,
                                 input_name);
 }
+OM_uint32 gss_get_name_attribute(OM_uint32 *minor_status,
+                                 gss_name_t name,
+                                 gss_buffer_t attr,
+                                 int *authenticated,
+                                 int *complete,
+                                 gss_buffer_t value,
+                                 gss_buffer_t display_value,
+                                 int *more)
+{
+    return gssntlm_get_name_attribute(minor_status, name, attr, authenticated,
+                                      complete, value, display_value, more);
+}
 
 OM_uint32 gss_context_time(OM_uint32 *minor_status,
                            gss_ctx_id_t context_handle,
@@ -418,3 +415,49 @@ OM_uint32 gss_inquire_name(OM_uint32 *minor_status,
     return gssntlm_inquire_name(minor_status, name, name_is_MN, MN_mech,
                                 attrs);
 }
+
+OM_uint32 gss_inquire_saslname_for_mech(OM_uint32 *minor_status,
+                                        const gss_OID desired_mech,
+                                        gss_buffer_t sasl_mech_name,
+                                        gss_buffer_t mech_name,
+                                        gss_buffer_t mech_description)
+{
+    return gssntlm_inquire_saslname_for_mech(minor_status, desired_mech,
+                                             sasl_mech_name, mech_name,
+                                             mech_description);
+}
+
+OM_uint32 gss_inquire_mech_for_saslname(OM_uint32 *minor_status,
+                                        const gss_buffer_t sasl_mech_name,
+                                        gss_OID *mech_type)
+{
+    return gssntlm_inquire_mech_for_saslname(minor_status, sasl_mech_name,
+                                             mech_type);
+}
+
+OM_uint32 gss_inquire_attrs_for_mech(OM_uint32 *minor_status,
+                                     gss_const_OID mech_oid,
+                                     gss_OID_set *mech_attrs,
+                                     gss_OID_set *known_mech_attrs)
+{
+    return gssntlm_inquire_attrs_for_mech(minor_status, mech_oid, mech_attrs,
+                                          known_mech_attrs);
+}
+
+OM_uint32 gssspi_mech_invoke(OM_uint32 *minor_status,
+                             const gss_OID desired_mech,
+                             const gss_OID desired_object,
+                             gss_buffer_t value)
+{
+    return gssntlm_mech_invoke(minor_status, desired_mech, desired_object,
+                               value);
+}
+
+OM_uint32 gssspi_set_cred_option(OM_uint32 *minor_status,
+                                 gss_cred_id_t *cred_handle,
+                                 const gss_OID desired_object,
+                                 const gss_buffer_t value)
+{
+    return gssntlm_set_cred_option(minor_status, cred_handle, desired_object,
+                                   value);
+}
