@@ -1,20 +1,4 @@
-/*
-   Copyright (C) 2013 Simo Sorce <simo@samba.org>
-
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Lesser General Public
-   License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Lesser General Public License for more details.
-
-   You should have received a copy of the GNU Lesser General Public
-   License along with this library; if not, see <http://www.gnu.org/licenses/>.
-*/
-
+/* Copyright 2013 Simo Sorce <simo@samba.org>, see COPYING for license */
 
 /* This File implements the NTLM protocol as specified by:
  *      [MS-NLMP]: NT LAN Manager (NTLM) Authentication Protocol
@@ -68,12 +52,6 @@ struct wire_single_host_data {
 };
 #pragma pack(pop)
 
-#pragma pack(push, 1)
-struct wire_channel_binding {
-    uint8_t md5_hash[16];
-};
-#pragma pack(pop)
-
 #pragma pack(push, 1)
 struct wire_ntlm_cli_chal {
     uint8_t resp_type;
@@ -100,12 +78,12 @@ int ntlm_init_ctx(struct ntlm_ctx **ctx)
     _ctx = calloc(1, sizeof(struct ntlm_ctx));
     if (!_ctx) return ENOMEM;
 
-    _ctx->from_oem = iconv_open("UCS-2LE", "UTF-8");
+    _ctx->from_oem = iconv_open("UTF16LE", "UTF-8");
     if (_ctx->from_oem == (iconv_t) -1) {
         ret = errno;
     }
 
-    _ctx->to_oem = iconv_open("UTF-8", "UCS-2LE");
+    _ctx->to_oem = iconv_open("UTF-8", "UTF16LE");
     if (_ctx->to_oem == (iconv_t) -1) {
         iconv_close(_ctx->from_oem);
         ret = errno;
@@ -189,7 +167,7 @@ bool ntlm_casecmp(const char *s1, const char *s2)
 
 /**
  * @brief  Converts a string using the provided iconv context.
- *         This function is ok only to convert utf8<->ucs2
+ *         This function is ok only to convert utf8<->utf16le
  *
  * @param cd        The iconv context
  * @param in        Input buffer
@@ -227,7 +205,6 @@ static int ntlm_str_convert(iconv_t cd,
     return 0;
 }
 
-
 uint8_t ntlmssp_sig[8] = {'N', 'T', 'L', 'M', 'S', 'S', 'P', 0};
 
 static void ntlm_encode_header(struct wire_msg_hdr *hdr, uint32_t msg_type)
@@ -278,6 +255,7 @@ static int ntlm_decode_oem_str(struct wire_field_hdr *str_hdr,
     str_offs = le32toh(str_hdr->offset);
     if ((str_offs < payload_offs) ||
         (str_offs > buffer->length) ||
+        (UINT32_MAX - str_offs < str_len) ||
         (str_offs + str_len > buffer->length)) {
         return ERR_DECODE;
     }
@@ -290,7 +268,7 @@ done:
     return 0;
 }
 
-static int ntlm_encode_ucs2_str_hdr(struct ntlm_ctx *ctx,
+static int ntlm_encode_u16l_str_hdr(struct ntlm_ctx *ctx,
                                     struct wire_field_hdr *hdr,
                                     struct ntlm_buffer *buffer,
                                     size_t *data_offs,
@@ -313,7 +291,7 @@ static int ntlm_encode_ucs2_str_hdr(struct ntlm_ctx *ctx,
     return 0;
 }
 
-static int ntlm_decode_ucs2_str_hdr(struct ntlm_ctx *ctx,
+static int ntlm_decode_u16l_str_hdr(struct ntlm_ctx *ctx,
                                     struct wire_field_hdr *str_hdr,
                                     struct ntlm_buffer *buffer,
                                     size_t payload_offs, char **str)
@@ -321,7 +299,7 @@ static int ntlm_decode_ucs2_str_hdr(struct ntlm_ctx *ctx,
     char *in, *out = NULL;
     uint16_t str_len;
     uint32_t str_offs;
-    size_t outlen;
+    size_t outlen = 0;
     int ret = 0;
 
     str_len = le16toh(str_hdr->len);
@@ -330,6 +308,7 @@ static int ntlm_decode_ucs2_str_hdr(struct ntlm_ctx *ctx,
     str_offs = le32toh(str_hdr->offset);
     if ((str_offs < payload_offs) ||
         (str_offs > buffer->length) ||
+        (UINT32_MAX - str_offs < str_len) ||
         (str_offs + str_len > buffer->length)) {
         return ERR_DECODE;
     }
@@ -341,13 +320,14 @@ static int ntlm_decode_ucs2_str_hdr(struct ntlm_ctx *ctx,
 
     ret = ntlm_str_convert(ctx->to_oem, in, out, str_len, &outlen);
 
-    /* make sure to terminate output string */
-    out[outlen] = '\0';
-
 done:
     if (ret) {
         safefree(out);
+    } else {
+        /* make sure to terminate output string */
+        out[outlen] = '\0';
     }
+
     *str = out;
     return ret;
 }
@@ -371,15 +351,14 @@ void ntlm_internal_set_version(uint8_t major, uint8_t minor,
 
 static int ntlm_encode_version(struct ntlm_ctx *ctx,
                                struct ntlm_buffer *buffer,
-                               size_t *data_offs)
+                               size_t data_offs)
 {
-    if (*data_offs + sizeof(struct wire_version) > buffer->length) {
+    if (data_offs + sizeof(struct wire_version) > buffer->length) {
         return ERR_ENCODE;
     }
 
-    memcpy(&buffer->data[*data_offs], &ntlmssp_version,
+    memcpy(&buffer->data[data_offs], &ntlmssp_version,
            sizeof(struct wire_version));
-    *data_offs += sizeof(struct wire_version);
     return 0;
 }
 
@@ -416,6 +395,7 @@ static int ntlm_decode_field(struct wire_field_hdr *hdr,
     offs = le32toh(hdr->offset);
     if ((offs < payload_offs) ||
         (offs > buffer->length) ||
+        (UINT32_MAX - offs < len) ||
         (offs + len > buffer->length)) {
         return ERR_DECODE;
     }
@@ -431,7 +411,7 @@ done:
     return 0;
 }
 
-static int ntlm_encode_av_pair_ucs2_str(struct ntlm_ctx *ctx,
+static int ntlm_encode_av_pair_u16l_str(struct ntlm_ctx *ctx,
                                         struct ntlm_buffer *buffer,
                                         size_t *data_offs,
                                         enum msv_av_ids av_id,
@@ -459,7 +439,7 @@ static int ntlm_encode_av_pair_ucs2_str(struct ntlm_ctx *ctx,
     return 0;
 }
 
-static int ntlm_decode_av_pair_ucs2_str(struct ntlm_ctx *ctx,
+static int ntlm_decode_av_pair_u16l_str(struct ntlm_ctx *ctx,
                                         struct wire_av_pair *av_pair,
                                         char **str)
 {
@@ -560,7 +540,7 @@ int ntlm_encode_target_info(struct ntlm_ctx *ctx, char *nb_computer_name,
         av_target_name_len = strlen(av_target_name);
         max_size += 4 + av_target_name_len * 2;
     }
-    if (av_cb) {
+    if (av_cb && av_cb->length > 0) {
         max_size += 4 + av_cb->length;
     }
 
@@ -570,35 +550,35 @@ int ntlm_encode_target_info(struct ntlm_ctx *ctx, char *nb_computer_name,
     if (!buffer.data) return ENOMEM;
 
     if (nb_computer_name) {
-        ret = ntlm_encode_av_pair_ucs2_str(ctx, &buffer, &data_offs,
+        ret = ntlm_encode_av_pair_u16l_str(ctx, &buffer, &data_offs,
                                            MSV_AV_NB_COMPUTER_NAME,
                                            nb_computer_name,
                                            nb_computer_name_len);
         if (ret) goto done;
     }
     if (nb_domain_name) {
-        ret = ntlm_encode_av_pair_ucs2_str(ctx, &buffer, &data_offs,
+        ret = ntlm_encode_av_pair_u16l_str(ctx, &buffer, &data_offs,
                                            MSV_AV_NB_DOMAIN_NAME,
                                            nb_domain_name,
                                            nb_domain_name_len);
         if (ret) goto done;
     }
     if (dns_computer_name) {
-        ret = ntlm_encode_av_pair_ucs2_str(ctx, &buffer, &data_offs,
+        ret = ntlm_encode_av_pair_u16l_str(ctx, &buffer, &data_offs,
                                            MSV_AV_DNS_COMPUTER_NAME,
                                            dns_computer_name,
                                            dns_computer_name_len);
         if (ret) goto done;
     }
     if (dns_domain_name) {
-        ret = ntlm_encode_av_pair_ucs2_str(ctx, &buffer, &data_offs,
+        ret = ntlm_encode_av_pair_u16l_str(ctx, &buffer, &data_offs,
                                            MSV_AV_DNS_DOMAIN_NAME,
                                            dns_domain_name,
                                            dns_domain_name_len);
         if (ret) goto done;
     }
     if (dns_tree_name) {
-        ret = ntlm_encode_av_pair_ucs2_str(ctx, &buffer, &data_offs,
+        ret = ntlm_encode_av_pair_u16l_str(ctx, &buffer, &data_offs,
                                            MSV_AV_DNS_TREE_NAME,
                                            dns_tree_name,
                                            dns_tree_name_len);
@@ -626,13 +606,13 @@ int ntlm_encode_target_info(struct ntlm_ctx *ctx, char *nb_computer_name,
         if (ret) goto done;
     }
     if (av_target_name) {
-        ret = ntlm_encode_av_pair_ucs2_str(ctx, &buffer, &data_offs,
+        ret = ntlm_encode_av_pair_u16l_str(ctx, &buffer, &data_offs,
                                            MSV_AV_TARGET_NAME,
                                            av_target_name,
                                            av_target_name_len);
         if (ret) goto done;
     }
-    if (av_cb) {
+    if (av_cb && av_cb->length > 0) {
         ret = ntlm_encode_av_pair_value(&buffer, &data_offs,
                                         MSV_AV_CHANNEL_BINDINGS, av_cb);
         if (ret) goto done;
@@ -695,7 +675,7 @@ int ntlm_decode_target_info(struct ntlm_ctx *ctx, struct ntlm_buffer *buffer,
             break;
         case MSV_AV_TARGET_NAME:
             if (!av_target_name) continue;
-            ret = ntlm_decode_av_pair_ucs2_str(ctx, av_pair, &av_target);
+            ret = ntlm_decode_av_pair_u16l_str(ctx, av_pair, &av_target);
             if (ret) goto done;
             break;
         case MSV_AV_SINGLE_HOST:
@@ -705,37 +685,45 @@ int ntlm_decode_target_info(struct ntlm_ctx *ctx, struct ntlm_buffer *buffer,
             break;
         case MSV_AV_TIMESTAMP:
             if (!av_timestamp) continue;
+            if (av_len < sizeof(timestamp)) {
+                ret = ERR_DECODE;
+                goto done;
+            }
             memcpy(&timestamp, av_pair->value, sizeof(timestamp));
             timestamp = le64toh(timestamp);
             break;
         case MSV_AV_FLAGS:
             if (!av_flags) continue;
+            if (av_len < sizeof(flags)) {
+                ret = ERR_DECODE;
+                goto done;
+            }
             memcpy(&flags, av_pair->value, sizeof(flags));
             flags = le32toh(flags);
             break;
         case MSV_AV_DNS_TREE_NAME:
             if (!dns_tree_name) continue;
-            ret = ntlm_decode_av_pair_ucs2_str(ctx, av_pair, &dns_tree);
+            ret = ntlm_decode_av_pair_u16l_str(ctx, av_pair, &dns_tree);
             if (ret) goto done;
             break;
         case MSV_AV_DNS_DOMAIN_NAME:
             if (!dns_domain_name) continue;
-            ret = ntlm_decode_av_pair_ucs2_str(ctx, av_pair, &dns_domain);
+            ret = ntlm_decode_av_pair_u16l_str(ctx, av_pair, &dns_domain);
             if (ret) goto done;
             break;
         case MSV_AV_DNS_COMPUTER_NAME:
             if (!dns_computer_name) continue;
-            ret = ntlm_decode_av_pair_ucs2_str(ctx, av_pair, &dns_computer);
+            ret = ntlm_decode_av_pair_u16l_str(ctx, av_pair, &dns_computer);
             if (ret) goto done;
             break;
         case MSV_AV_NB_DOMAIN_NAME:
             if (!nb_domain_name) continue;
-            ret = ntlm_decode_av_pair_ucs2_str(ctx, av_pair, &nb_domain);
+            ret = ntlm_decode_av_pair_u16l_str(ctx, av_pair, &nb_domain);
             if (ret) goto done;
             break;
         case MSV_AV_NB_COMPUTER_NAME:
             if (!nb_computer_name) continue;
-            ret = ntlm_decode_av_pair_ucs2_str(ctx, av_pair, &nb_computer);
+            ret = ntlm_decode_av_pair_u16l_str(ctx, av_pair, &nb_computer);
             if (ret) goto done;
             break;
         default:
@@ -751,8 +739,6 @@ int ntlm_decode_target_info(struct ntlm_ctx *ctx, struct ntlm_buffer *buffer,
 
 done:
     if (ret) {
-        ntlm_free_buffer_data(&sh);
-        ntlm_free_buffer_data(&cb);
         safefree(nb_computer);
         safefree(nb_domain);
         safefree(dns_computer);
@@ -776,7 +762,7 @@ done:
 
 int ntlm_process_target_info(struct ntlm_ctx *ctx, bool protect,
                              struct ntlm_buffer *in,
-                             const char *server,
+                             const char *spn,
                              struct ntlm_buffer *unhashed_cb,
                              struct ntlm_buffer *out,
                              uint64_t *out_srv_time,
@@ -791,7 +777,7 @@ int ntlm_process_target_info(struct ntlm_ctx *ctx, bool protect,
     uint32_t av_flags = 0;
     uint64_t srv_time = 0;
     uint8_t cb[16] = { 0 };
-    struct ntlm_buffer av_cb = { cb, 16 };
+    struct ntlm_buffer av_cb = { NULL, 0 };
     int ret = 0;
 
     /* TODO: check that returned netbios/dns names match ? */
@@ -808,8 +794,9 @@ int ntlm_process_target_info(struct ntlm_ctx *ctx, bool protect,
         goto done;
     }
 
-    if (server && av_target_name) {
-        if (strcasecmp(server, av_target_name) != 0) {
+    if (spn && av_target_name &&
+        ((av_flags & MSVAVFLAGS_UNVERIFIED_SPN) == 0)) {
+        if (strcasecmp(spn, av_target_name) != 0) {
             ret = EINVAL;
             goto done;
         }
@@ -824,19 +811,19 @@ int ntlm_process_target_info(struct ntlm_ctx *ctx, bool protect,
     }
 
     if (unhashed_cb->length > 0) {
+        av_cb.data = cb;
+        av_cb.length = 16;
         ret = ntlm_hash_channel_bindings(unhashed_cb, &av_cb);
         if (ret) goto done;
     }
 
-    if (!av_target_name && server) {
-        av_target_name = strdup(server);
+    if (!av_target_name && spn) {
+        av_target_name = strdup(spn);
         if (!av_target_name) {
             ret = ENOMEM;
             goto done;
         }
     }
-    /* TODO: add way to tell if the target name is verified o not,
-     * if not set av_flags |= MSVAVFLAGS_UNVERIFIED_SPN; */
 
     ret = ntlm_encode_target_info(ctx,
                                   nb_computer_name, nb_domain_name,
@@ -952,6 +939,12 @@ int ntlm_encode_neg_msg(struct ntlm_ctx *ctx, uint32_t flags,
         if (ret) goto done;
     }
 
+    if (flags & NTLMSSP_NEGOTIATE_VERSION) {
+        ret = ntlm_encode_version(ctx, &buffer,
+                                  (char *)&msg->version - (char *)msg);
+        if (ret) goto done;
+    }
+
 done:
     if (ret) {
         safefree(buffer.data);
@@ -979,6 +972,14 @@ int ntlm_decode_neg_msg(struct ntlm_ctx *ctx,
 
     neg_flags = le32toh(msg->neg_flags);
 
+    if ((neg_flags & NTLMSSP_NEGOTIATE_VERSION) == 0) {
+        /* adjust the payload offset to point to the
+         * version field, for compatibility with older
+         * clients that completely omitted the structure
+         * on the wire */
+        payload_offs -= sizeof(struct wire_version);
+    }
+
     if (domain &&
         (neg_flags & NTLMSSP_NEGOTIATE_OEM_DOMAIN_SUPPLIED)) {
         ret = ntlm_decode_oem_str(&msg->domain_name, buffer,
@@ -1004,7 +1005,6 @@ done:
     return ret;
 }
 
-/* TODO: support datagram style */
 int ntlm_encode_chal_msg(struct ntlm_ctx *ctx,
                          uint32_t flags,
                          const char *target_name,
@@ -1024,10 +1024,6 @@ int ntlm_encode_chal_msg(struct ntlm_ctx *ctx,
 
     buffer.length = sizeof(struct wire_chal_msg);
 
-    if (flags & NTLMSSP_NEGOTIATE_VERSION) {
-        buffer.length += sizeof(struct wire_version);
-    }
-
     if ((flags & NTLMSSP_TARGET_TYPE_SERVER)
         || (flags & NTLMSSP_TARGET_TYPE_DOMAIN)) {
         if (!target_name) return EINVAL;
@@ -1056,14 +1052,15 @@ int ntlm_encode_chal_msg(struct ntlm_ctx *ctx,
 
     /* this must be first as it pushes the payload further down */
     if (flags & NTLMSSP_NEGOTIATE_VERSION) {
-        ret = ntlm_encode_version(ctx, &buffer, &data_offs);
+        ret = ntlm_encode_version(ctx, &buffer,
+                                  (char *)&msg->version - (char *)msg);
         if (ret) goto done;
     }
 
     if ((flags & NTLMSSP_TARGET_TYPE_SERVER)
         || (flags & NTLMSSP_TARGET_TYPE_DOMAIN)) {
         if (flags & NTLMSSP_NEGOTIATE_UNICODE) {
-            ret = ntlm_encode_ucs2_str_hdr(ctx, &msg->target_name, &buffer,
+            ret = ntlm_encode_u16l_str_hdr(ctx, &msg->target_name, &buffer,
                                            &data_offs, target_name, target_len);
         } else {
             ret = ntlm_encode_oem_str(&msg->target_name, &buffer,
@@ -1098,6 +1095,7 @@ int ntlm_decode_chal_msg(struct ntlm_ctx *ctx,
 {
     struct wire_chal_msg *msg;
     size_t payload_offs;
+    size_t base_chal_size;
     uint32_t flags;
     char *trg = NULL;
     int ret = 0;
@@ -1110,11 +1108,21 @@ int ntlm_decode_chal_msg(struct ntlm_ctx *ctx,
     payload_offs = (char *)msg->payload - (char *)msg;
 
     flags = le32toh(msg->neg_flags);
+    base_chal_size = sizeof(struct wire_chal_msg);
+
+    if ((flags & NTLMSSP_NEGOTIATE_VERSION) == 0) {
+        /* adjust the payload offset to point to the
+         * version field, for compatibility with older
+         * clients that completely omitted the structure
+         * on the wire */
+        payload_offs -= sizeof(struct wire_version);
+        base_chal_size -= sizeof(struct wire_version);
+    }
 
     if ((flags & NTLMSSP_TARGET_TYPE_SERVER)
         || (flags & NTLMSSP_TARGET_TYPE_DOMAIN)) {
         if (flags & NTLMSSP_NEGOTIATE_UNICODE) {
-            ret = ntlm_decode_ucs2_str_hdr(ctx, &msg->target_name, buffer,
+            ret = ntlm_decode_u16l_str_hdr(ctx, &msg->target_name, buffer,
                                            payload_offs, &trg);
         } else {
             ret = ntlm_decode_oem_str(&msg->target_name, buffer,
@@ -1128,7 +1136,7 @@ int ntlm_decode_chal_msg(struct ntlm_ctx *ctx,
 
     /* if we allowed a broken short challenge message from an old
      * server we must stop here */
-    if (buffer->length < sizeof(struct wire_chal_msg)) {
+    if (buffer->length < base_chal_size) {
         if (flags & NTLMSSP_NEGOTIATE_TARGET_INFO) {
             ret = ERR_DECODE;
         }
@@ -1211,9 +1219,6 @@ int ntlm_encode_auth_msg(struct ntlm_ctx *ctx,
     if (enc_sess_key) {
         buffer.length += enc_sess_key->length;
     }
-    if (flags & NTLMSSP_NEGOTIATE_VERSION) {
-        buffer.length += sizeof(struct wire_version);
-    }
     if (mic) {
         buffer.length += 16;
     }
@@ -1228,11 +1233,12 @@ int ntlm_encode_auth_msg(struct ntlm_ctx *ctx,
 
     /* this must be first as it pushes the payload further down */
     if (flags & NTLMSSP_NEGOTIATE_VERSION) {
-        ret = ntlm_encode_version(ctx, &buffer, &data_offs);
+        ret = ntlm_encode_version(ctx, &buffer,
+                                  (char *)&msg->version - (char *)msg);
         if (ret) goto done;
     }
 
-    /* this must be second as it pushes the payload further down */
+    /* this pushes the payload further down */
     if (mic) {
         memset(&buffer.data[data_offs], 0, mic->length);
         /* return the actual pointer back in the mic, as it will
@@ -1251,7 +1257,7 @@ int ntlm_encode_auth_msg(struct ntlm_ctx *ctx,
 
     if (domain_name_len) {
         if (flags & NTLMSSP_NEGOTIATE_UNICODE) {
-            ret = ntlm_encode_ucs2_str_hdr(ctx, &msg->domain_name,
+            ret = ntlm_encode_u16l_str_hdr(ctx, &msg->domain_name,
                                            &buffer, &data_offs,
                                            domain_name, domain_name_len);
         } else {
@@ -1263,7 +1269,7 @@ int ntlm_encode_auth_msg(struct ntlm_ctx *ctx,
     }
     if (user_name_len) {
         if (flags & NTLMSSP_NEGOTIATE_UNICODE) {
-            ret = ntlm_encode_ucs2_str_hdr(ctx, &msg->user_name,
+            ret = ntlm_encode_u16l_str_hdr(ctx, &msg->user_name,
                                            &buffer, &data_offs,
                                            user_name, user_name_len);
         } else {
@@ -1275,7 +1281,7 @@ int ntlm_encode_auth_msg(struct ntlm_ctx *ctx,
     }
     if (workstation_len) {
         if (flags & NTLMSSP_NEGOTIATE_UNICODE) {
-            ret = ntlm_encode_ucs2_str_hdr(ctx, &msg->workstation,
+            ret = ntlm_encode_u16l_str_hdr(ctx, &msg->workstation,
                                            &buffer, &data_offs,
                                            workstation, workstation_len);
         } else {
@@ -1314,6 +1320,7 @@ int ntlm_decode_auth_msg(struct ntlm_ctx *ctx,
                          struct ntlm_buffer *mic)
 {
     struct wire_auth_msg *msg;
+    uint32_t neg_flags;
     size_t payload_offs;
     char *dom = NULL;
     char *usr = NULL;
@@ -1329,10 +1336,13 @@ int ntlm_decode_auth_msg(struct ntlm_ctx *ctx,
     msg = (struct wire_auth_msg *)buffer->data;
     payload_offs = (char *)msg->payload - (char *)msg;
 
-    /* this must be first as it pushes the payload further down */
-    if (flags & NTLMSSP_NEGOTIATE_VERSION) {
-        /* skip version for now */
-        payload_offs += sizeof(struct wire_version);
+    neg_flags = le32toh(msg->neg_flags);
+    if ((neg_flags & NTLMSSP_NEGOTIATE_VERSION) == 0) {
+        /* adjust the payload offset to point to the
+         * version field, for compatibility with older
+         * clients that completely omitted the structure
+         * on the wire */
+        payload_offs -= sizeof(struct wire_version);
     }
 
     /* Unconditionally copy 16 bytes for the MIC, if it was really
@@ -1340,12 +1350,24 @@ int ntlm_decode_auth_msg(struct ntlm_ctx *ctx,
      * in the NT Response, that will be fully decoded later by the caller
      * and the MIC checked otherwise these 16 bytes will just be ignored */
     if (mic) {
+        size_t mic_offs = payload_offs;
+
         if (mic->length < 16) return ERR_DECODE;
-        /* mic is at payload_offs right now */
-        if (buffer->length - payload_offs < 16) return ERR_DECODE;
-        memcpy(mic->data, &buffer->data[payload_offs], 16);
-        /* NOTE: we do not push down the payload because we do not know that
-         * the MIC is actually present yet for real */
+
+        if ((neg_flags & NTLMSSP_NEGOTIATE_VERSION) == 0) {
+            struct wire_version zver = {0};
+            /* mic is at payload_offs right now, but this offset may be
+             * wrongly reduced for compatibility with older clients,
+             * if all bytes are zeroed, then it means there was an actual
+             * empty version struct */
+            if (memcmp(&msg->version, &zver,
+                       sizeof(struct wire_version)) == 0) {
+                mic_offs += sizeof(struct wire_version);
+            }
+        }
+
+        if (buffer->length - mic_offs < 16) return ERR_DECODE;
+        memcpy(mic->data, &buffer->data[mic_offs], 16);
     }
 
     if (msg->lm_chalresp.len != 0 && lm_chalresp) {
@@ -1381,7 +1403,7 @@ int ntlm_decode_auth_msg(struct ntlm_ctx *ctx,
     }
     if (msg->domain_name.len != 0 && domain_name) {
         if (flags & NTLMSSP_NEGOTIATE_UNICODE) {
-            ret = ntlm_decode_ucs2_str_hdr(ctx, &msg->domain_name, buffer,
+            ret = ntlm_decode_u16l_str_hdr(ctx, &msg->domain_name, buffer,
                                            payload_offs, &dom);
         } else {
             ret = ntlm_decode_oem_str(&msg->domain_name, buffer,
@@ -1391,7 +1413,7 @@ int ntlm_decode_auth_msg(struct ntlm_ctx *ctx,
     }
     if (msg->user_name.len != 0 && user_name) {
         if (flags & NTLMSSP_NEGOTIATE_UNICODE) {
-            ret = ntlm_decode_ucs2_str_hdr(ctx, &msg->user_name, buffer,
+            ret = ntlm_decode_u16l_str_hdr(ctx, &msg->user_name, buffer,
                                            payload_offs, &usr);
         } else {
             ret = ntlm_decode_oem_str(&msg->user_name, buffer,
@@ -1401,7 +1423,7 @@ int ntlm_decode_auth_msg(struct ntlm_ctx *ctx,
     }
     if (msg->workstation.len != 0 && workstation) {
         if (flags & NTLMSSP_NEGOTIATE_UNICODE) {
-            ret = ntlm_decode_ucs2_str_hdr(ctx, &msg->workstation, buffer,
+            ret = ntlm_decode_u16l_str_hdr(ctx, &msg->workstation, buffer,
                                            payload_offs, &wks);
         } else {
             ret = ntlm_decode_oem_str(&msg->workstation, buffer,
