@@ -1,19 +1,4 @@
-/*
-   Copyright (C) 2013 Simo Sorce <simo@samba.org>
-
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Lesser General Public
-   License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Lesser General Public License for more details.
-
-   You should have received a copy of the GNU Lesser General Public
-   License along with this library; if not, see <http://www.gnu.org/licenses/>.
-*/
+/* Copyright 2013 Simo Sorce <simo@samba.org>, see COPYING for license */
 
 #ifndef _NTLM_H_
 #define _NTLM_H
@@ -59,9 +44,11 @@
 /* (2.2.2.10 VERSION) */
 #define WINDOWS_MAJOR_VERSION_5 0x05
 #define WINDOWS_MAJOR_VERSION_6 0x06
+#define WINDOWS_MAJOR_VERSION_10 0x0A
 #define WINDOWS_MINOR_VERSION_0 0x00
 #define WINDOWS_MINOR_VERSION_1 0x01
 #define WINDOWS_MINOR_VERSION_2 0x02
+#define WINDOWS_MINOR_VERSION_3 0x03
 #define NTLMSSP_REVISION_W2K3 0x0F
 
 #define NTLMSSP_VERSION_MAJOR WINDOWS_MAJOR_VERSION_6
@@ -355,6 +342,13 @@ int ntlm_reset_rc4_state(uint32_t flags, bool recv,
                          struct ntlm_key *session_key,
                          struct ntlm_signseal_state *state);
 
+/**
+ * @brief   Release the RC4 state
+ *
+ * @param signseal_state        Sign and seal keys and state
+ */
+void ntlm_release_rc4_state(struct ntlm_signseal_state *state);
+
 /**
  * @brief   Verifies a NTLM v1 NT Response
  *
