@@ -1,20 +1,4 @@
-/*
-   Copyright (C) 2013 Simo Sorce <simo@samba.org>
-
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Lesser General Public
-   License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Lesser General Public License for more details.
-
-   You should have received a copy of the GNU Lesser General Public
-   License along with this library; if not, see <http://www.gnu.org/licenses/>.
-*/
-
+/* Copyright 2013 Simo Sorce <simo@samba.org>, see COPYING for license */
 
 /* This File implements the NTLM protocol as specified by:
  *      [MS-NLMP]: NT LAN Manager (NTLM) Authentication Protocol
@@ -66,7 +50,7 @@ int NTOWFv1(const char *password, struct ntlm_key *result)
     int ret;
 
     len = strlen(password);
-    retstr = u8_conv_to_encoding("UCS-2LE", iconveh_error,
+    retstr = u8_conv_to_encoding("UTF16LE", iconveh_error,
                                  (const uint8_t *)password, len,
                                  NULL, NULL, &out);
     if (!retstr) return ERR_CRYPTO;
@@ -96,7 +80,10 @@ int LMOWFv1(const char *password, struct ntlm_key *result)
     if (result->length != 16) return EINVAL;
 
     len = strlen(password);
-    if (len > 14) return ERANGE;
+    if (len > 14) {
+        memset(result->data, 0, result->length);
+        return 0;
+    }
 
     out = 15;
     retstr = (char *)u8_toupper((const uint8_t *)password, len,
@@ -267,7 +254,7 @@ int NTOWFv2(struct ntlm_ctx *ctx, struct ntlm_key *nt_hash,
         offs += len;
     }
 
-    retstr = (uint8_t *)u8_conv_to_encoding("UCS-2LE", iconveh_error,
+    retstr = (uint8_t *)u8_conv_to_encoding("UTF16LE", iconveh_error,
                                             upcased, offs, NULL, NULL, &out);
     if (!retstr) return ERR_CRYPTO;
 
@@ -590,6 +577,12 @@ int ntlm_reset_rc4_state(uint32_t flags, bool recv,
     return ret;
 }
 
+void ntlm_release_rc4_state(struct ntlm_signseal_state *state)
+{
+    RC4_FREE(&state->recv.seal_handle);
+    RC4_FREE(&state->send.seal_handle);
+}
+
 static int ntlm_seal_regen(struct ntlm_signseal_handle *h)
 {
     struct ntlm_buffer payload;
@@ -847,8 +840,8 @@ int ntlm_sign(uint32_t flags, int direction,
         return 0;
 
     } else if (flags & NTLMSSP_NEGOTIATE_ALWAYS_SIGN) {
-        uint32_t le_seq = htole32(h->seq_num);
-        memcpy(signature->data, &le_seq, 4);
+        uint32_t sig_ver = htole32(NTLMSSP_MESSAGE_SIGNATURE_VERSION);
+        memcpy(signature->data, &sig_ver, 4);
         memset(&signature->data[4], 0, 12);
         return 0;
     }
@@ -867,6 +860,10 @@ int ntlm_seal(uint32_t flags,
 
     h = &state->send;
 
+    if (h->seal_handle == NULL) {
+        return EINVAL;
+    }
+
     ret = RC4_UPDATE(h->seal_handle, message, output);
     if (ret) return ret;
 
@@ -904,6 +901,10 @@ int ntlm_unseal(uint32_t flags,
         h = &state->recv;
     }
 
+    if (h->seal_handle == NULL) {
+        return EINVAL;
+    }
+
     ret = RC4_UPDATE(h->seal_handle, message, output);
     if (ret) return ret;
 
@@ -969,9 +970,17 @@ int ntlm_verify_mic(struct ntlm_key *key,
 
     /* flags must be checked as they may push the payload further down */
     flags = le32toh(msg->neg_flags);
-    if (flags & NTLMSSP_NEGOTIATE_VERSION) {
-        /* skip version for now */
-        payload_offs += sizeof(struct wire_version);
+    if ((flags & NTLMSSP_NEGOTIATE_VERSION) == 0) {
+        struct wire_version zver = {0};
+        /* mic is at payload_offs right now, but this offset may
+         * need to be reduced if the sender completely omitted
+         * the version struct, as some older clients do */
+        if (memcmp(&msg->version, &zver,
+                   sizeof(struct wire_version)) != 0) {
+            /* version struct is not all zeros, this indicates the actual
+             * struct was omitted and payload is shifted down */
+            payload_offs -= sizeof(struct wire_version);
+        }
     }
 
     if (payload_offs + NTLM_SIGNATURE_SIZE > authenticate_message->length) {
